# smah/settings/system/operating_system/__init__.py
from .operating_system import OperatingSystem
from .configurator import operating_system_terminal_configurator
__all__ = ['OperatingSystem', 'operating_system_terminal_configurator']